package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.EXCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.LOWER_BOUNDS;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.IntegerBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.LongBoundary;
import java.util.Optional;

public class BiCreaterLowerBounds extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterLowerBounds instance = new BiCreaterLowerBounds();

  private BiCreaterLowerBounds() {
    super();
  }

  public static BiCreaterLowerBounds getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return LOWER_BOUNDS;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1 instanceof IntegerBoundary) {
      if ((int) b1.getLowerBound() + 1 == (int) b2.getLowerBound()
          && b1.getLowerBoundType() == EXCLUSIVE
          && b2.getLowerBoundType() == INCLUSIVE) {
        return Optional.empty();
      }
    }
    if (b1 instanceof LongBoundary) {
      if (b1.getUpperBound().equals((long) b2.getUpperBound() + 1)
          && b1.getLowerBoundType() == EXCLUSIVE
          && b2.getLowerBoundType() == INCLUSIVE) {
        return Optional.empty();
      }
    }
    try {
      return Optional.of(
          b1.getCopy(
              b1.getLowerBound(),
              b2.getLowerBound(),
              b1.getLowerBoundType().getOp(),
              b2.getLowerBoundType()));
    } catch (Exception e) {
      return Optional.empty();
    }
  }
}
