package de.unikoblenz.fgbks.base.utils.boundary.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundType;
import de.unikoblenz.fgbks.base.utils.boundary.BoundaryParseException;

public class BooleanBoundary extends AbstractGrowingBoundary<Boolean> {

  private static final String TRUE_STRING = "true";
  private static final String FALSE_STRING = "false";

  public BooleanBoundary(String text) {
    super(text, Boolean.TYPE);
  }

  private BooleanBoundary() {
    super(Boolean.TYPE);
  }

  @Override
  public Boolean getMinValue() {
    return false;
  }

  @Override
  public Boolean getMaxValue() {
    return true;
  }

  @Override
  public AbstractGrowingBoundary getCopy(
      Boolean lowerBound, Boolean upperBound, BoundType lowerBoundType, BoundType upperBoundType) {
    BooleanBoundary v = new BooleanBoundary();
    v.lowerBound = lowerBound;
    v.upperBound = upperBound;
    v.lowerBoundType = lowerBoundType;
    v.upperBoundType = upperBoundType;
    v.validate();
    return v;
  }

  @Override
  protected void parse(String text) {
    upperBoundType = INCLUSIVE;
    lowerBoundType = INCLUSIVE;
    if (text.trim().isEmpty()) {
      lowerBound = getMinValue();
      upperBound = getMaxValue();
    } else if (TRUE_STRING.equals(text.toLowerCase())) {
      lowerBound = upperBound = getMaxValue();
    } else if (FALSE_STRING.equals(text.toLowerCase())) {
      lowerBound = upperBound = getMinValue();
    } else {
      throw new BoundaryParseException("Boolean value cloud not be parsed: " + text);
    }
  }
}
